@@ -1,6 +1,23 @@
-"""a Python NATS client"""
+"""a Python NATS client
 
-__version__ = "2022.3.0"
+Checkout `if __name__ == "__main__"` block for usage
+
+$ python -m goingnats
+--- one ---
+Message(subject=b'time.time', payload=b'1646389270.6568809')
+--- client.subscribe + client.request ---
+Message(subject=b'time.time', payload=b'1646389271.6775382')
+Message(subject=b'time.time', payload=b'1646389272.690817')
+Message(subject=b'time.time', payload=b'1646389273.70843')
+Response(payload=b'20220304')
+Message(subject=b'time.time', payload=b'1646389274.724187')
+Message(subject=b'time.time', payload=b'1646389275.733218')
+--- request ---
+Response(payload=b'6')
+no respponse received from b'today' in 100 ms
+"""
+
+__version__ = "2022.3.2"
 
 import json
 import queue
@@ -74,7 +91,7 @@ class Client:
         except TypeError:
             raise TypeError("subject must be bytes-like") from None
 
-    def request(self, *, subject, payload=b""):
+    def request(self, *, subject, payload=b"", wait=None):
         """request subject for a response to payload"""
         inbox = f"INBOX.{uuid.uuid4().hex}".encode()
         self._sid += 1
@@ -96,7 +113,10 @@ class Client:
             )
         except TypeError:
             raise TypeError("subject and payload must be bytes-like") from None
-        return self._response.get()
+        try:
+            return self._response.get(timeout=wait / 1_000 if wait else None)
+        except queue.Empty:
+            raise TimeoutError(f"no respponse received from {subject} in {wait} ms")
 
     def __enter__(self):
         try:
@@ -130,7 +150,7 @@ class Client:
 
     def _thread(self):
         # https://docs.nats.io/nats-protocol/nats-protocol
-        buffer_ = b""
+        messages = Messages()
         while self._run:
             try:
                 received = self._sock.recv(4096)
@@ -139,29 +159,17 @@ class Client:
             except OSError as e:
                 if self._run:
                     raise OSError("recv failed") from e
-
-            segments = received.split(b"\r\n")
-            if len(segments) > 1:
-                # buffer contains tail of previous received \r\ntail
-                segments[0] = buffer_ + segments[0]
-                # will be b"" if received end with \r\n
-                buffer_ = segments[-1]
-            else:
-                # no \r\n received nothing to do but buffer
-                buffer_ += received
-                continue
-
-            for segment in segments[:-1]:
-                if segment == b"PING":
+            for message in messages(received):
+                if message == b"PING":
                     self._send([b"PONG"])
-                elif segment == b"PONG":
+                elif message == b"PONG":
                     pass
-                elif segment == b"+OK":
+                elif message == b"+OK":
                     pass
-                elif segment.startswith(b"-ERR"):
-                    warnings.warn(segment.decode())
-                elif segment.startswith(b"INFO"):
-                    self.information = json.loads(segment[4:])
+                elif message.startswith(b"-ERR"):
+                    warnings.warn(message.decode())
+                elif message.startswith(b"INFO"):
+                    self.information = json.loads(message[4:])
                     information = {
                         "name": self.name,
                         "verbose": False,
@@ -174,9 +182,9 @@ class Client:
                             json.dumps(information).encode(),
                         ]
                     )
-                elif segment.startswith(b"MSG"):
+                elif message.startswith(b"MSG"):
                     # MSG ...  \r\n[payload]\r\n
-                    split = segment.split(b" ")
+                    split = message.split(b" ")
                     subject = split[1]
                     # is it a request?
                     inbox = False
@@ -186,13 +194,35 @@ class Client:
                     # should be e a payload
                     # request
                     if inbox:
-                        self._messages.put(Request(subject, inbox, segment))
+                        self._messages.put(Request(subject, inbox, message))
                     # response
                     elif subject.startswith(b"INBOX."):
-                        self._response.put(Response(segment))
+                        self._response.put(Response(message))
                     # vanilla message
                     else:
-                        self._messages.put(Message(subject, segment))
+                        self._messages.put(Message(subject, message))
+
+
+class Messages:
+    def __init__(self, separator=CRLF):
+        self.separator = separator
+        self._buffer = b""
+
+    def __call__(self, received):
+        self._buffer = b"".join([self._buffer, received])
+        return self
+
+    def __iter__(self):
+        messages = self._buffer.split(self.separator)
+        try:
+            last = self._buffer[-1]
+        except IndexError:
+            return
+        if last != self.separator:
+            self._buffer = messages[-1]
+            messages = messages[:-1]
+        for message in messages:
+            yield (message)
 
 
 Message = namedtuple("Message", "subject payload")
@@ -200,7 +230,7 @@ Request = namedtuple("Request", "subject inbox payload")
 Response = namedtuple("Response", "payload")
 
 
-def one(*, subject, host="0.0.0.0", port=4222, name="one"):
+def one(*, subject, host="127.0.0.1", port=4222, name="one"):
     """returns one (the first) message received on a subject"""
     with Client(host=host, port=port, name=name) as client:
         client.subscribe(subject=subject)
@@ -209,6 +239,11 @@ def one(*, subject, host="0.0.0.0", port=4222, name="one"):
                 return message
 
 
+def request(*, subject, payload=b"", wait=None, host="127.0.0.1", port=4222):
+    with Client(host=host, port=port, name=f"request.{subject.decode()}") as client:
+        return client.request(subject=subject, payload=payload, wait=wait)
+
+
 def _int_to_bytes(i):
     return f"{i}".encode()
 
@@ -230,23 +265,31 @@ if __name__ == "__main__":
         """respond to request for today with the date"""
         with Client(name="responder") as client:
             client.subscribe(subject=b"today")
+            client.subscribe(subject=b"add")
             while True:
                 for request in client.get():
-                    if request.subject != b"today":
+                    if request.subject == b"today":
+                        # slow responder
+                        time.sleep(2)
+                        # will format the date according to payload or defaults to ...
+                        format = request.payload.decode() if request.payload else "%Y-%m-%d"
+                        response = f"{dt.date.today():{format}}".encode()
+                    elif request.subject == b"add":
+                        response = _int_to_bytes(sum(json.loads(request.payload)))
+                    else:
                         continue
-                    # slow responder
-                    time.sleep(2)
-                    # will format the date according to payload or defaults to ...
-                    format = request.payload.decode() if request.payload else "%Y-%m-%d"
                     client.publish(
                         subject=request.inbox,
-                        payload=f"{dt.date.today():{format}}".encode(),
+                        payload=response,
                     )
 
     threading.Thread(target=responder, daemon=True).start()
 
     # application
     with Client(name="consumer") as client:
+        print("--- one ---")
+        print(one(subject=b"time.time"))
+        print("--- client.subscribe + client.request ---")
         client.subscribe(subject=b"time.time")
         received = 0
         response = None
@@ -259,3 +302,9 @@ if __name__ == "__main__":
                 # request response are blocking
                 response = client.request(subject=b"today", payload=b"%Y%m%d")
                 print(response)
+        print("--- request ---")
+        print(request(subject=b"add", payload=b"[1, 2, 3]"))
+        try:
+            print(request(subject=b"today", wait=100))
+        except TimeoutError as e:
+            print(e)
